import 'dart:async';
import 'dart:convert';
import 'dart:math';



import 'package:tencent_im_base/tencent_im_base.dart';
import 'package:tim_ui_kit_calling_plugin/model/TIMUIKitCallingListener.dart';
import 'package:tim_ui_kit_calling_plugin/model/TIMUIKitInvitedParams.dart';

import '../TRTCCalling.dart';
import '../TRTCCallingDef.dart';
import '../TRTCCallingDelegate.dart';

//trtc sdk
import 'package:tencent_trtc_cloud/trtc_cloud.dart';
import 'package:tencent_trtc_cloud/trtc_cloud_def.dart';
import 'package:tencent_trtc_cloud/tx_audio_effect_manager.dart';
import 'package:tencent_trtc_cloud/tx_device_manager.dart';
import 'package:tencent_trtc_cloud/tx_beauty_manager.dart';

// import 'package:tim_ui_kit_plugin_interface/tim_ui_kit_plugin_interface.dart';

// im sdk


class CallingMessage {
  /// 发起邀请方
  String? inviter;

  /// 被邀请方
  List<String>? inviteeList;

  /// 0: 语音 1: 视频
  int? callType;

  // 1: 邀请方发起邀请
  // 2: 邀请方取消邀请
  // 3: 被邀请方接受邀请
  // 4: 被邀请方拒绝邀请
  // 5: 邀请超时
  int? actionType;

  /// 邀请ID
  String? inviteID;

  /// 通话时间
  int? timeout;

  /// 通话房间
  int? roomID;

  // 通话时间：秒，大于0代表通话时间
  int? callEnd;

  CallingMessage({
    this.inviter,
    this.actionType,
    this.inviteID,
    this.inviteeList,
    this.timeout,
    // this.roomID,
    // this.callType,
    // this.callEnd
  });

  CallingMessage.fromJSON(json) {
    actionType = json["actionType"];
    timeout = json["timeout"];
    inviter = json["inviter"];
    inviteeList = List<String>.from(json["inviteeList"]);
    inviteID = json["inviteID"];
    // callType = jsonDecode(json["data"])["call_type"];
    // roomID = jsonDecode(json["data"])["room_id"];
    // callEnd = jsonDecode(json["data"])["call_end"];
  }

  toJson() {
    final Map<String, dynamic> data = {};
    data["actionType"] = actionType;

    data["timeout"] = timeout;
    data["inviter"] = inviter;
    data["inviteeList"] = inviteeList;
    data["inviteID"] = inviteID;
    // data["inviter"] = jsonDecode(json["data"])["call_type"];
    // data["inviter"] = jsonDecode(json["data"])["room_id"];
    // data["call_end"] = callEnd;

    return data;
  }
}

class TRTCCallingImpl extends TRTCCalling {
  String logTag = "TRTCCallingImpl";
  static TRTCCallingImpl? sInstance;

  int timeOutCount = 30; //超时时间，默认30s

  int codeErr = -1;

  // 是否首次邀请
  bool isOnCalling = false;
  bool mIsInRoom = false;
  int mEnterRoomTime = 0;
  String mCurCallID = "";
  int mCurRoomID = 0;
  String mCurGroupId = ""; //当前群组通话的群组ID
  String? mNickName;
  String? mFaceUrl;

  /*
   * 当前邀请列表
   * C2C通话时会记录自己邀请的用户
   * IM群组通话时会同步群组内邀请的用户
   * 当用户接听、拒绝、忙线、超时会从列表中移除该用户
   */
  List<String> mCurInvitedList = [];

  List<dynamic> mCurCallList = [];

  //当前语音通话中的远端用户
  Set mCurRoomRemoteUserSet = {};
  /*
  * C2C通话的邀请人
  * 例如A邀请B，B存储的mCurSponsorForMe为A
  */
  String mCurSponsorForMe = "";
  //当前通话的类型 对应native的中的cmd
  String? mCurCallType;

  late int mSdkAppId;
  late String mCurUserId;
  late String mCurUserSig;
  String? mRoomId;
  String? mOwnerUserId;
  bool mIsInitIMSDK = false;
  bool mIsLogin = false;
  bool mIsTRTCCloudReady = false;
  String mRole = "audience"; //默认为观众，archor为主播
  late V2TIMManager timManager;
  late TRTCCloud mTRTCCloud;
  late TXAudioEffectManager txAudioManager;
  late TXDeviceManager txDeviceManager;
  Set<TUICallingListener> listeners = {};

  TRTCCallingImpl() {
    //获取腾讯即时通信IM manager
    timManager = TencentImSDKPlugin.v2TIMManager;
    initTRTC();
  }

  initTRTC() async {
    mTRTCCloud = (await TRTCCloud.sharedInstance())!;
    txDeviceManager = mTRTCCloud.getDeviceManager();
    txAudioManager = mTRTCCloud.getAudioEffectManager();
    mIsTRTCCloudReady = true;
  }

  String getDurationTimeString(Duration duration) {
    String line = "";
    if (duration.inHours != 0) {
      line = twoDigits(duration.inHours.remainder(24)) + ":";
    }
    line = line + twoDigits(duration.inMinutes.remainder(60)) + ":";
    line = line + twoDigits(duration.inSeconds.remainder(60));
    return line;
  }

  String twoDigits(int n) {
    if (n >= 10) return "$n";
    return "0$n";
  }

  int getSeconds(int millonSeconds) {
    int seconds = (millonSeconds / 1000).ceil();

    return seconds;
  }

  static sharedInstance() {
    sInstance ??= TRTCCallingImpl();
    return sInstance;
  }

  static void destroySharedInstance() {
    if (sInstance != null) {
      sInstance = null;
    }
    TRTCCloud.destroySharedInstance();
  }

  @override
  void destroy() {
    mTRTCCloud.stopLocalPreview();
    mTRTCCloud.stopLocalAudio();
    mTRTCCloud.exitRoom();
  }

  @override
  void registerListener(TUICallingListener func) async {
    if (listeners.isEmpty) {
      listeners.add(func);
      //监听im事件
      timManager
          .getSignalingManager()
          .addSignalingListener(listener: signalingListener());
      if (!mIsTRTCCloudReady) {
        mTRTCCloud = (await TRTCCloud.sharedInstance())!;
      }
      //监听trtc事件
      mTRTCCloud.registerListener(rtcListener);
    } else {
      listeners.add(func);
    }
  }

  @override
  void unRegisterListener(TUICallingListener func) {
    listeners.remove(func);
    if (listeners.isEmpty) {
      mTRTCCloud.unRegisterListener(rtcListener);
      timManager.getSignalingManager().removeSignalingListener();
    }
  }

  emitEvent(TRTCCallingDelegate type, params) {
    for (var item in listeners) {
      switch (type) {
        case TRTCCallingDelegate.onInvited:
          final onInvitedPrams = InviteParams(
              callType: params["type"],
              groupID: params["groupID"],
              isFromGroup: params["isFromGroup"],
              userIDList: params["userIds"],
              sponsor: params["sponsor"]);
          item.onInvited(onInvitedPrams);
          break;
        case TRTCCallingDelegate.onCallEnd:
          item.onCallEnd();
          break;
        case TRTCCallingDelegate.onCallingCancel:
          item.onCallingCancel();
          break;
        case TRTCCallingDelegate.onUserEnter:
          final userEnterParams = OnUserEnterParams(userID: params);
          item.onUserEnter(userEnterParams);
          break;
        case TRTCCallingDelegate.onCallingTimeout:
          item.onCallingTimeout(OnCallingTimeoutParams(userIDList: List<String>.from(params)));
          break;
        case TRTCCallingDelegate.onLineBusy:
          final onLineBusyParams = OnLineBusyParams(userID: params);
          item.onLineBusy(onLineBusyParams);
          break;
        case TRTCCallingDelegate.onNoResp:
          final onNoRespParams = NoRespParams(userID: params);
          item.onNoResp(onNoRespParams);
          break;
        case TRTCCallingDelegate.onUserVideoAvailable:
          final onUserVideoAvailableParams = OnUserVideoAvailableParams(
              userID: params["userId"], available: params["available"]);
          item.onUserVideoAvailable(onUserVideoAvailableParams);
          break;
        case TRTCCallingDelegate.onUserAudioAvailable:
          final onUserAudioAvailableParams = OnUserAudioAvailableParams(
              userID: params["userId"], available: params["available"]);
          item.onUserAudioAvailable(onUserAudioAvailableParams);
          break;
        case TRTCCallingDelegate.onKickedOffline:
          item.onKickedOffline();
          break;
        case TRTCCallingDelegate.onUserLeave:
          final leaveParams = OnUserLeaveParams(
              userID: params["userId"], reason: params["reason"]);
          item.onUserLeave(leaveParams);
          break;
        case TRTCCallingDelegate.onEnterRoom:
          item.onEnterRoom(params);
          break;
        case TRTCCallingDelegate.onWarning:
          final warningPrams = OnWarningPrams(
              warningCode: params["warningCode"],
              warningMsg: params["warningMsg"],
              extraInfo: params["extraInfo"]);
          item.onWarning(warningPrams);
          break;
        case TRTCCallingDelegate.onError:
          final errprParams = OnErrorParams(
              errCode: params["errCode"],
              errMsg: params["errMsg"],
              extraInfo: params["extraInfo"]);
          item.onError(errprParams);
          break;
        case TRTCCallingDelegate.onReject:
          final rejectParams = RejectParams(userID: params);
          item.onReject(rejectParams);
          break;
        case TRTCCallingDelegate.onGroupCallInviteeListUpdate:
          final groupCallListUpdate =
              OnGroupCallInviteeListUpdateParams(userIDList: params);
          item.onGroupCallInviteeListUpdate(groupCallListUpdate);
          break;
        case TRTCCallingDelegate.onUserVoiceVolume:
          item.onUserVoiceVolume(params);
          break;
        case TRTCCallingDelegate.onAccept:
          final acceptParams = AcceptParams(userID: params);
          item.onAccept(acceptParams);
          break;
      }
    }
  }

  V2TimSignalingListener signalingListener() {
    TRTCCallingDelegate type;
    return V2TimSignalingListener(
      onInvitationCancelled: (inviteID, inviter, data) {
        if (!_isCallingData(data)) {
          return;
        }
        if (inviteID == mCurCallID) {
          _stopCall();
          emitEvent(TRTCCallingDelegate.onCallingCancel, {});
        }
      },
      onInvitationTimeout: (inviteID, inviteeList) {
        //邀请者
        String curGroupCallId = _getGroupCallId(mCurUserId);
        if (!_isEmpty(mCurCallID) && inviteID != mCurCallID) {
          return;
        } else if (_isEmpty(mCurCallID) && inviteID != curGroupCallId) {
          return;
        }
        if (mCurSponsorForMe.isEmpty) {
          for (var i = 0; i < inviteeList.length; i++) {
            emitEvent(TRTCCallingDelegate.onNoResp, inviteeList[i]);
            mCurInvitedList.remove(inviteeList[i]);
          }
        } else {

          //被邀请者
          if (inviteeList.contains(mCurUserId)) {
            _stopCall();
            emitEvent(TRTCCallingDelegate.onCallingTimeout, []);
          } else {
            emitEvent(TRTCCallingDelegate.onCallingTimeout, inviteeList);
          }
          // ignore: list_remove_unrelated_type
          mCurInvitedList.remove(inviteeList);
        }
        // 每次超时都需要判断当前是否需要结束通话
        _preExitRoom(null);
      },
      onInviteeAccepted: (inviteID, invitee, data) {
        if (!_isCallingData(data)) {
          return;
        }
        mCurInvitedList.remove(invitee);
        emitEvent(TRTCCallingDelegate.onAccept,invitee);
      },
      onInviteeRejected: (inviteID, invitee, data) {
        if (!_isCallingData(data)) {
          return;
        }
        String curGroupCallId = _getGroupCallId(invitee);
        if (mCurCallID == inviteID || curGroupCallId == inviteID) {
          try {
            Map<String, dynamic>? customMap = _callingDataAdapt(data);
            mCurInvitedList.remove(invitee);
            if (customMap != null &&
                (customMap.containsKey('message') ||
                    customMap.containsKey("line_busy"))) {
              final message = customMap["line_busy"] ?? customMap["message"];
              if (message == "lineBusy") {
                emitEvent(TRTCCallingDelegate.onLineBusy, invitee);
              }
            } else {
              emitEvent(TRTCCallingDelegate.onReject, invitee);
            }
            _preExitRoom(null);
          } catch (e) {
            print(logTag +
                "=onInviteeRejected JsonSyntaxException:" +
                e.toString());
          }
        }
      },
      onReceiveNewInvitation:
          (inviteID, inviter, groupID, inviteeList, data) async {
        if (!_isCallingData(data)) {
          return;
        }
        try {
          Map<String, dynamic>? customMap = _callingDataAdapt(data);
          if (customMap == null) {
            print(logTag + "onReceiveNewInvitation extraMap is null, ignore");
            return;
          }
          if (customMap.containsKey('cmd') ||
              customMap.containsKey("call_type")) {
            if (customMap.containsKey('cmd')) {
              mCurCallType = customMap['cmd'];
            } else {
              mCurCallType = customMap["call_type"] == 1
                  ? TRTCCalling.typeAudioCall
                  : TRTCCalling.typeVideoCall;
            }
          }
          if (customMap.containsKey('call_end')) {
            _preExitRoom(null);
            return;
          }
          if (customMap.containsKey('room_id')) {
            mCurRoomID = customMap['room_id'];
          }
        } catch (e) {
          print(logTag +
              "=onReceiveNewInvitation JsonSyntaxException:" +
              e.toString());
        }
        if (isOnCalling && inviteeList.contains(mCurUserId)) {
          // 正在通话时，收到了一个邀请我的通话请求,需要告诉对方忙线
          Map<String, dynamic> busyMap = getCustomMap();
          // message为native那边的额外业务字段
          busyMap["data"]['message'] = 'lineBusy';
          busyMap["line_busy"] = 'lineBusy';
          await timManager
              .getSignalingManager()
              .reject(inviteID: inviteID, data: jsonEncode(busyMap));
          return;
        }
        // 与对方处在同一个群中，此时收到了邀请群中其他人通话的请求，界面上展示连接动画
        if (!_isEmpty(groupID) && !_isEmpty(mCurGroupId)) {
          mCurInvitedList.addAll(inviteeList);
          TRTCCallingDelegate type =
              TRTCCallingDelegate.onGroupCallInviteeListUpdate;
          emitEvent(type, mCurInvitedList);
        }
        if (!inviteeList.contains(mCurUserId)) {
          return;
        }

        mCurSponsorForMe = inviter;
        mCurCallID = inviteID;
        mCurGroupId = groupID;
        type = TRTCCallingDelegate.onInvited;
        inviteeList.remove(mCurUserId);
        emitEvent(type, {
          'sponsor': inviter,
          'userIds': inviteeList,
          'isFromGroup': !_isEmpty(groupID),
          'type': mCurCallType,
          'groupID': groupID
        });
      },
    );
  }

  _callingDataAdapt(String data) {
    final customMap = jsonDecode(data);
    final isNewSignaling = customMap["data"] != null;
    if (isNewSignaling) {
      if (customMap["call_end"] != null) {
        customMap["data"]["call_end"] = customMap["call_end"];
      }
      return customMap["data"];
    }
    return customMap;
  }

  rtcListener(rtcType, param) {
    String typeStr = rtcType.toString();
    TRTCCallingDelegate type;
    typeStr = typeStr.replaceFirst("TRTCCloudListener.", "");
    if (typeStr == "onEnterRoom") {
      if (param < 0) {
        _stopCall();
      } else {
        mIsInRoom = true;
      }
      type = TRTCCallingDelegate.onEnterRoom;
      emitEvent(type, param);
    } else if (typeStr == "onError") {
      type = TRTCCallingDelegate.onError;
      emitEvent(type, param);
    } else if (typeStr == "onUserVoiceVolume") {
      type = TRTCCallingDelegate.onUserVoiceVolume;
      emitEvent(type, param);
    } else if (typeStr == "onUserVideoAvailable") {
      type = TRTCCallingDelegate.onUserVideoAvailable;
      emitEvent(type, param);
    } else if (typeStr == "onUserAudioAvailable") {
      type = TRTCCallingDelegate.onUserAudioAvailable;
      emitEvent(type, param);
    } else if (typeStr == "onRemoteUserEnterRoom") {
      mCurRoomRemoteUserSet.add(param);
      //todo 这里需要测试一下群组
      if (mCurRoomRemoteUserSet.length == 1) {
        mEnterRoomTime = DateTime.now().millisecondsSinceEpoch;
      }

      type = TRTCCallingDelegate.onUserEnter;
      emitEvent(type, param);
    } else if (typeStr == "onRemoteUserLeaveRoom") {
      mCurRoomRemoteUserSet.remove(param['userId']);
      mCurInvitedList.remove(param['userId']);
      type = TRTCCallingDelegate.onUserLeave;
      emitEvent(type, param);
      _preExitRoom(param['userId']);
    }
  }

  // 多人通话时，根据userId找到对应的通话id
  _getGroupCallId(String userId) {
    for (int i = 0; i < mCurCallList.length; i++) {
      if (mCurCallList[i]['userId'] == userId) {
        return mCurCallList[i]['callId'];
      }
    }
    return '';
  }

  /*
  * 重要：用于判断是否需要结束本次通话
  * 在用户超时、拒绝、忙线、有人退出房间时需要进行判断
  */
  _preExitRoom(String? leaveUser) async {
    if (mCurRoomRemoteUserSet.isEmpty && mCurInvitedList.isEmpty && mIsInRoom) {
      // 当没有其他用户在房间里了，则结束通话。
      if (!_isEmpty(leaveUser)) {
        var time = DateTime.now().millisecondsSinceEpoch - mEnterRoomTime;
        // 挂断时添加时间
        Map<String, dynamic> customMap = getCustomMap();
        customMap['call_end'] = getSeconds(time);
        // 通话结束信令提醒
        if (_isEmpty(mCurGroupId)) {
          timManager.getSignalingManager().invite(
              invitee: leaveUser!, data: jsonEncode(customMap), timeout: 0);
        } else {
          timManager.getSignalingManager().inviteInGroup(
              groupID: mCurGroupId,
              inviteeList: [leaveUser!],
              data: jsonEncode(customMap),
              timeout: 0);
        }
      }
      _exitRoom();
      _stopCall();
      emitEvent(TRTCCallingDelegate.onCallEnd, {});
    }
  }

  @override
  Future<ActionCallback> login(
      int sdkAppId, String userId, String userSig) async {
    mSdkAppId = sdkAppId;
    mCurUserId = userId;
    mCurUserSig = userSig;

    if (!mIsInitIMSDK) {
      //初始化SDK
      V2TimValueCallback<bool> initRes = await timManager.initSDK(
          sdkAppID: sdkAppId, //填入在控制台上申请的sdkappid
          loglevel: LogLevelEnum.V2TIM_LOG_ERROR,
          listener: V2TimSDKListener(onKickedOffline: () {
            TRTCCallingDelegate type = TRTCCallingDelegate.onKickedOffline;
            emitEvent(type, {});
          }));
      if (initRes.code != 0) {
        //初始化sdk错误
        return ActionCallback(code: 0, desc: 'init im sdk error');
      }
    }
    mIsInitIMSDK = true;

    // 登陆到 IM
    String? loginedUserId = (await timManager.getLoginUser()).data;

    if (loginedUserId != null && loginedUserId == userId) {
      mIsLogin = true;
      return ActionCallback(code: 0, desc: 'login im success');
    }
    V2TimCallback loginRes =
        await timManager.login(userID: userId, userSig: userSig);
    if (loginRes.code == 0) {
      mIsLogin = true;
      return ActionCallback(code: 0, desc: 'login im success');
    } else {
      return ActionCallback(code: codeErr, desc: loginRes.desc);
    }
  }

  @override
  Future<ActionCallback> logout() async {
    V2TimCallback loginRes = await timManager.logout();
    _stopCall();
    _exitRoom();
    mNickName = "";
    mFaceUrl = "";
    return ActionCallback(code: loginRes.code, desc: loginRes.desc);
  }

  @override
  Future<ActionCallback> call(
      String userId, String type, OfflinePushInfo? offlinePushInfo) async {
    if (!isOnCalling) {
      // 首次拨打电话，生成id，并进入trtc房间
      mCurRoomID = _generateRoomID();
      mCurCallType = type;
      _enterTRTCRoom();
    }
    mCurInvitedList.add(userId);

    V2TimValueCallback res = await timManager.getSignalingManager().invite(
        invitee: userId,
        data: jsonEncode(getCustomMap()),
        timeout: timeOutCount,
        offlinePushInfo: offlinePushInfo,
        onlineUserOnly: false);
    if(res.code == 0){
      mCurCallID = res.data;
      mCurCallList.add({'userId': userId, 'callId': mCurCallID});
      return ActionCallback(code: res.code, desc: res.desc);
    }
    return ActionCallback(code: res.code, desc: res.desc);
  }

  _isCallingData(String data) {
    try {
      Map<String, dynamic> customMap = jsonDecode(data);
      if (customMap.containsKey('call_type')) {
        return true;
      }
    } catch (e) {
      print("isCallingData json parse error");
      return false;
    }
    return false;
  }

  getCustomMap({bool isGroup = false}) {
    Map<String, dynamic> customMap = <String, dynamic>{};
    Map<String, dynamic> data = <String, dynamic>{};
    data['cmd'] = mCurCallType;
    data['room_id'] = mCurRoomID;
    data["message"] = "";
    data["cmd_info"] = "";
    customMap['version'] = 1;
    customMap['call_type'] = mCurCallType == TRTCCalling.typeAudioCall ? 1 : 2;
    customMap['data'] = data;
    customMap['room_id'] = mCurRoomID;
    customMap['is_group'] = isGroup;
    return customMap;
  }

  /*
  * trtc 进房
  */
  _enterTRTCRoom() async {
    isOnCalling = true;
    if (mCurCallType == TRTCCalling.typeVideoCall) {
      // 开启基础美颜
      TXBeautyManager txBeautyManager = mTRTCCloud.getBeautyManager();
      // 自然美颜
      txBeautyManager.setBeautyStyle(1);
      txBeautyManager.setBeautyLevel(6);
      // 进房前需要设置一下关键参数
      TRTCVideoEncParam encParam = TRTCVideoEncParam();
      encParam.videoResolution = TRTCCloudDef.TRTC_VIDEO_RESOLUTION_960_540;
      encParam.videoFps = 15;
      encParam.videoBitrate = 1000;
      encParam.videoResolutionMode =
          TRTCCloudDef.TRTC_VIDEO_RESOLUTION_MODE_PORTRAIT;
      encParam.enableAdjustRes = true;
      mTRTCCloud.setVideoEncoderParam(encParam);
    }

    mTRTCCloud.enableAudioVolumeEvaluation(500);
    txDeviceManager.setAudioRoute(TRTCCloudDef.TRTC_AUDIO_ROUTE_SPEAKER);
    mTRTCCloud.muteLocalAudio(false);
    mTRTCCloud.startLocalAudio(TRTCCloudDef.TRTC_AUDIO_QUALITY_DEFAULT);
    mTRTCCloud.callExperimentalAPI(
        "{\"api\": \"setFramework\", \"params\": {\"framework\": 7, \"component\": 3}}");
    mTRTCCloud.enterRoom(
        TRTCParams(
            sdkAppId: mSdkAppId,
            userId: mCurUserId,
            userSig: mCurUserSig,
            roomId: mCurRoomID,
            role: TRTCCloudDef.TRTCRoleAnchor),
        mCurCallType == TRTCCalling.typeVideoCall
            ? TRTCCloudDef.TRTC_APP_SCENE_VIDEOCALL
            : TRTCCloudDef.TRTC_APP_SCENE_AUDIOCALL);
  }

  @override
  Future<ActionCallback> groupCall(
      List<String> userIdList, String type, String? groupId) async {
    if (_isListEmpty(userIdList)) {
      return ActionCallback(code: codeErr, desc: 'userIdList is empty');
    }
    if (!isOnCalling) {
      // 首次拨打电话，生成id，并进入trtc房间
      mCurRoomID = _generateRoomID();
      mCurCallType = type;
      _enterTRTCRoom();
    }

    // 过滤已经邀请的用户id
    List<String> filterInvitedList = [];
    for (var i = 0; i < userIdList.length; i++) {
      if (!mCurInvitedList.contains(userIdList[i])) {
        filterInvitedList.add(userIdList[i]);
      }
    }
    if (_isListEmpty(filterInvitedList)) {
      return ActionCallback(
          code: codeErr, desc: 'the userIdList has been invited');
    }
    mCurInvitedList = filterInvitedList;
    if (_isEmpty(groupId)) {
      var customMap = getCustomMap(isGroup: true);

      for (int i = 0; i < mCurInvitedList.length; i++) {
        V2TimValueCallback res = await timManager.getSignalingManager().invite(
            invitee: mCurInvitedList[i],
            data: jsonEncode(customMap),
            timeout: timeOutCount,
            onlineUserOnly: false);
            if(res.code == 0){
              mCurCallList.add({'userId': mCurInvitedList[i], 'callId': res.data});
            }
        
      }
      return ActionCallback(code: 0, desc: '');
    } else {
      mCurGroupId = groupId!;
      var customMap = getCustomMap(isGroup: true);
      V2TimValueCallback res = await timManager
          .getSignalingManager()
          .inviteInGroup(
              groupID: groupId,
              inviteeList: mCurInvitedList,
              data: jsonEncode(customMap),
              timeout: timeOutCount,
              onlineUserOnly: false);
              if(res.code == 0){
                mCurCallID = res.data;
      return ActionCallback(code: res.code, desc: res.desc);
              }
      return ActionCallback(code: res.code, desc: res.desc);
    }
  }

  _isListEmpty(List? list) {
    return list == null || list.length == 0;
  }

  _isEmpty(String? data) {
    return data == null || data == "";
  }

  /*
  * 停止此次通话，把所有的变量都会重置
  */
  _stopCall() {
    isOnCalling = false;
    mIsInRoom = false;
    mEnterRoomTime = 0;
    mCurCallID = "";
    mCurRoomID = 0;
    mCurInvitedList = [];
    mCurCallList = [];
    mCurRoomRemoteUserSet.clear();
    mCurSponsorForMe = "";
    mCurGroupId = "";
    mCurCallType = TRTCCalling.typeUnknow;
  }

  @override
  Future<ActionCallback> accept() async {
    _enterTRTCRoom();
    V2TimCallback res = await timManager
        .getSignalingManager()
        .accept(inviteID: mCurCallID, data: jsonEncode(getCustomMap()));
    return ActionCallback(code: res.code, desc: res.desc);
  }

  @override
  Future<void> closeCamera() async {
    return mTRTCCloud.stopLocalPreview();
  }

  @override
  Future<void> hangup() async {
    if (!isOnCalling) {
      await reject();
      return;
    }
    _exitRoom();
    if (_isEmpty(mCurGroupId)) {
      for (int i = 0; i < mCurInvitedList.length; i++) {
        var time = DateTime.now().millisecondsSinceEpoch - mEnterRoomTime;
        var customMap = getCustomMap();
        customMap["call_end"] = time;
        await timManager.getSignalingManager().cancel(
            inviteID: _getGroupCallId(mCurInvitedList[i]),
            data: jsonEncode(customMap));
      }
    } else {
      if (mCurRoomRemoteUserSet.isEmpty) {
        timManager
            .getSignalingManager()
            .cancel(inviteID: mCurCallID, data: jsonEncode(getCustomMap()));
      }
    }
    _stopCall();
  }

  /*
  * trtc 退房
  */
  _exitRoom() {
    mTRTCCloud.stopLocalPreview();
    mTRTCCloud.stopLocalAudio();
    mTRTCCloud.exitRoom();
  }

  @override
  Future<void> openCamera(bool isFrontCamera, int viewId) {
    return mTRTCCloud.startLocalPreview(isFrontCamera, viewId);
  }

  @override
  Future<void> updateLocalView(int viewId) {
    return mTRTCCloud.updateLocalView(viewId);
  }

  @override
  Future<ActionCallback> reject() async {
    V2TimCallback res = await timManager
        .getSignalingManager()
        .reject(inviteID: mCurCallID, data: jsonEncode(getCustomMap()));
    _stopCall();
    return ActionCallback(code: res.code, desc: res.desc);
  }

  @override
  Future<void> setHandsFree(bool isHandsFree) {
    if (isHandsFree) {
      return txDeviceManager
          .setAudioRoute(TRTCCloudDef.TRTC_AUDIO_ROUTE_SPEAKER);
    } else {
      return txDeviceManager
          .setAudioRoute(TRTCCloudDef.TRTC_AUDIO_ROUTE_EARPIECE);
    }
  }

  @override
  Future<void> setMicMute(bool isMute) {
    return mTRTCCloud.muteLocalAudio(isMute);
  }

  @override
  Future<void> startRemoteView(String userId, int streamType, int viewId) {
    return mTRTCCloud.startRemoteView(userId, streamType, viewId);
  }

  @override
  Future<void> stopRemoteView(String userId, int streamType) {
    return mTRTCCloud.stopRemoteView(userId, streamType);
  }

  @override
  Future<void> updateRemoteView(String userId, int streamType, int viewId) {
    return mTRTCCloud.updateRemoteView(userId, streamType, viewId);
  }

  @override
  Future<void> switchCamera(bool isFrontCamera) {
    return txDeviceManager.switchCamera(isFrontCamera);
  }

  _generateRoomID() {
    Random rng = Random();
    //2147483647
    String numStr = '';
    for (var i = 0; i < 9; i++) {
      numStr += rng.nextInt(9).toString();
    }
    return int.tryParse(numStr);
  }

  @override
  checkIsSelf(String userID) {
    return userID == mCurUserId;
  }
}

/// @nodoc
typedef VoiceListenerFunc<P> = void Function(
    TRTCCallingDelegate type, P params);
