import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:tencent_im_base/tencent_im_base.dart';
import 'package:tencent_trtc_cloud/trtc_cloud_def.dart';
import 'package:tencent_trtc_cloud/trtc_cloud_video_view.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_call_conv_type.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_calling_scence.dart';
import 'package:tim_ui_kit_calling_plugin/model/TIMUIKitCallingListener.dart';
import 'package:tim_ui_kit_calling_plugin/model/TIMUIKitInvitedParams.dart';
import 'package:tim_ui_kit_calling_plugin/model/TRTCCalling.dart';
import 'package:tim_ui_kit_calling_plugin/model/TRTCCallingDef.dart';
import 'package:tim_ui_kit_calling_plugin/view/c2c/tim_uikit_trtc_c2c_extent_button.dart';
import 'package:tim_ui_kit_calling_plugin/view/group/tim_uikit_trtc_group_calling_tools.dart';
import 'package:tim_ui_kit_calling_plugin/view/group/tim_uikt_group_call_time.dart';
// import 'package:tim_ui_kit_plugin_interface/tim_ui_kit_plugin_interface.dart';

class UserViewInfo {
  bool isJoined;
  V2TimUserFullInfo userInfo;
  bool isPeaking;
  bool isMuteAudio;
  bool isMuteVideo;
  bool isSelf;
  int viewID;
  UserViewInfo(
      {required this.userInfo,
      this.isPeaking = false,
      this.isSelf = false,
      this.isMuteAudio = false,
      this.viewID = 0,
      this.isJoined = false,
      this.isMuteVideo = true});
}

class BtnItem {
  String label;
  String id;
  String iconPath;

  BtnItem({this.label = "", this.iconPath = "", this.id = ""});
}

enum BtnListType { videoCall, audioCall, ringCall, videoCallBottom }

class TIMUIKitGroupCalling extends StatefulWidget {
  final bool isInitiationCalling;
  final ConvType convType;
  final V2TimUserFullInfo inviterUserInfo;
  final List<V2TimUserFullInfo> inviteUsersInfoList;
  final CallingScenes callScenes;
  final String? groupID;
  final Function close;

  const TIMUIKitGroupCalling(
      {Key? key,
      required this.isInitiationCalling,
      required this.convType,
      required this.inviterUserInfo,
      required this.inviteUsersInfoList,
      required this.callScenes,
      this.groupID,
      required this.close})
      : super(key: key);
  @override
  State<StatefulWidget> createState() => _TIMUIKitGroupCalling();
}

class _TIMUIKitGroupCalling extends State<TIMUIKitGroupCalling> {
  late TRTCCalling _tRTCCallingService;

  List<UserViewInfo> userInfoList = [];
  bool isOpenMico = true;
  bool isOpenCamera = true;
  bool isCalling = false;
  bool isOpenHandsFree = true;
  int selfViewId = 0;
  bool _isFrontCamera = true;
  String callingTime = "00:00";
  late DateTime answerTime;
  late TUICallingListener _onRtcListener;

  _exitRoom() {}

  Widget _buildUserVideoItem(UserViewInfo item) {
    final isMuteVideo = item.isMuteVideo;
    return !isMuteVideo
        ? TRTCCloudVideoView(
            viewType: TRTCCloudDef.TRTC_VideoView_TextureView,
            onViewCreated: (viewId) {
              if (item.isSelf) {
                _tRTCCallingService.openCamera(_isFrontCamera, viewId);
                selfViewId = viewId;
              } else {
                var streamType = TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG;
                _tRTCCallingService.startRemoteView(
                    item.userInfo.userID!, streamType, viewId);
              }
              item.viewID = viewId;
            },
          )
        : Container();
  }

  List<BtnItem> _getBtnList(BtnListType type) {
    final micBtn = BtnItem(
        iconPath: isOpenMico
            ? "images/microphone-on.png"
            : "images/microphone-off.png",
        label: TIM_t("麦克风"),
        id: "mic");
    final handsFreeBtn = BtnItem(
        iconPath: isOpenHandsFree
            ? "images/trtccalling_ic_handsfree_enable.png"
            : "images/trtccalling_ic_handsfree_disable.png",
        label: TIM_t("扬声器"),
        id: "handsFree");
    final cameraBtn = BtnItem(
        iconPath:
            isOpenCamera ? "images/camera-on.png" : "images/camera-off.png",
        label: TIM_t("摄像头"),
        id: "camera");
    final hangUpBtn =
        BtnItem(iconPath: "images/hangup.png", label: TIM_t("挂断"), id: "hangUp");
    final acceptBtn = BtnItem(
        iconPath: "images/trtccalling_ic_dialing_pressed.png",
        label: TIM_t("接听"),
        id: "accept");

    final switchCamerBtn = BtnItem(
        iconPath: "images/switch-camera.png",
        label: TIM_t("切换摄像头"),
        id: "switchCamera");

    if (type == BtnListType.videoCall) {
      return [micBtn, handsFreeBtn, cameraBtn];
    }

    if (type == BtnListType.videoCallBottom) {
      return [hangUpBtn, switchCamerBtn];
    }

    if (type == BtnListType.audioCall) {
      return [micBtn, hangUpBtn, handsFreeBtn];
    }

    if (type == BtnListType.ringCall) {
      return [acceptBtn, hangUpBtn];
    }

    return [];
  }

  Widget _buildScreenList() {
    final screenList = GroupCallTool.getScreenList(userInfoList);
    return LayoutBuilder(builder: ((context, constraints) {
      return Align(
        alignment: Alignment.topLeft,
        child: ListView.builder(
          scrollDirection: Axis.horizontal,
          itemCount: screenList.length,
          itemBuilder: ((context, index) {
            final screenItem = screenList[index];
            return SizedBox(
              width: constraints.maxWidth,
              height: constraints.maxHeight,
              // color: Colors.redAccent,
              child: Wrap(
                alignment: WrapAlignment.center,
                children: List.generate(
                  screenItem.length,
                  (index) {
                    final userInfo = screenItem[index].userInfo;
                    final faceUrl = userInfo.faceUrl;
                    final widgetSize =
                        Size(constraints.maxWidth, constraints.maxHeight);
                    Size size = GroupCallTool.getViewSize(
                        widgetSize, screenItem.length);
                    double width = size.width;
                    double height = size.height;
                    return Container(
                      width: width,
                      height: height,
                      decoration: BoxDecoration(
                        image: DecorationImage(
                          colorFilter: !screenItem[index].isJoined
                              ? ColorFilter.mode(Colors.black.withOpacity(0.6),
                                  BlendMode.dstATop)
                              : null,
                          image: NetworkImage(faceUrl ??
                              "https://imgcache.qq.com/qcloud/public/static//avatar3_100.20191230.png"),
                          fit: BoxFit.cover,
                        ),
                      ),
                      child: screenItem[index].isJoined
                          ? _buildUserVideoItem(screenItem[index])
                          : const SpinKitThreeBounce(
                              color: Colors.white,
                              size: 35,
                            ),
                    );
                  },
                ),
              ),
            );
          }),
        ),
      );
    }));
  }

  _handleBtnOnTap(String id) {
    if (id == "mic") {
      isOpenMico = !isOpenMico;
      _tRTCCallingService.setMicMute(!isOpenMico);
    }

    if (id == "handsFree") {
      isOpenHandsFree = !isOpenHandsFree;
      _tRTCCallingService.setHandsFree(isOpenHandsFree);
    }

    if (id == "camera") {
      isOpenCamera = !isOpenCamera;
      if (!isOpenCamera) {
        _tRTCCallingService.closeCamera();
      }
      userInfoList = userInfoList.map(
        (e) {
          if (e.isSelf) {
            e.isMuteVideo = !isOpenCamera;
          }
          return e;
        },
      ).toList();
    }

    if (id == "accept") {
      _tRTCCallingService.accept();
    }

    if (id == "hangUp") {
      _tRTCCallingService.hangup();
      Future.delayed(const Duration(microseconds: 100), () {
        widget.close();
      });
    }

    if (id == "switchCamera") {
      _tRTCCallingService.switchCamera(!_isFrontCamera);
      _isFrontCamera = !_isFrontCamera;
    }
    setState(() {});
  }

  Widget _buildCallingBtnList() {
    final isSignling = isCalling || widget.isInitiationCalling;
    final isVideoCall = widget.callScenes == CallingScenes.Video;
    return isSignling && isVideoCall
        ? Wrap(
            spacing: 30,
            children: _getBtnList(BtnListType.videoCall)
                .map((e) => ExtendButton(
                      imgUrl: e.iconPath,
                      tips: e.label,
                      onTap: () {
                        _handleBtnOnTap(e.id);
                      },
                    ))
                .toList())
        : const SizedBox(
            height: 72,
          );
  }

  Widget _buildSignlingBtn() {
    final isRing = !widget.isInitiationCalling && !isCalling;
    final isVideoCall = widget.callScenes == CallingScenes.Video;
    if (isRing) {
      final ringBtnList = _getBtnList(BtnListType.ringCall);
      return Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: List.generate(
            ringBtnList.length,
            (index) => ExtendButton(
                  imgUrl: ringBtnList[index].iconPath,
                  tips: ringBtnList[index].label,
                  imgHieght: 64,
                  onTap: () {
                    _handleBtnOnTap(ringBtnList[index].id);
                  },
                )),
      );
    }

    if (isVideoCall) {
      final videoCallBottomBtnList = _getBtnList(BtnListType.videoCallBottom);
      return Row(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.end,
        children: [
          const SizedBox(
            width: 25,
          ),
          ...List.generate(videoCallBottomBtnList.length, (index) {
            final item = videoCallBottomBtnList[index];
            return Padding(
              padding: const EdgeInsets.only(left: 16),
              child: ExtendButton(
                imgUrl: item.iconPath,
                tips: item.id == "hangUp" ? item.label : "",
                imgHieght: item.id == "hangUp" ? 64 : 25,
                onTap: () {
                  _handleBtnOnTap(item.id);
                },
              ),
            );
          })
        ],
      );
    } else {
      final audioCallBtnList = _getBtnList(BtnListType.audioCall);
      return Wrap(
        spacing: 50,
        children: List.generate(audioCallBtnList.length, (index) {
          final item = audioCallBtnList[index];
          return ExtendButton(
            imgUrl: item.iconPath,
            tips: item.label,
            imgHieght: item.id == "hangUp" ? 64 : 0,
            onTap: () {
              _handleBtnOnTap(item.id);
            },
          );
        }),
      );
    }
  }

  _buildCallingMemberInfo() {
    final isVideoCall = widget.callScenes == CallingScenes.Video;
    final allCallingMembers = [
      widget.inviterUserInfo,
      ...widget.inviteUsersInfoList
    ];
    userInfoList = allCallingMembers
        .map((item) => UserViewInfo(
            userInfo: item,
            isMuteVideo:
                !isVideoCall || !_tRTCCallingService.checkIsSelf(item.userID!),
            isSelf: _tRTCCallingService.checkIsSelf(item.userID!),
            isJoined: _tRTCCallingService.checkIsSelf(item.userID!)))
        .toList();
    setState(() {});
  }

  Widget _buildBottomBtn() {
    final isVideoCall = widget.callScenes == CallingScenes.Video;
    return Column(
      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
      children: [
        SizedBox(
          height: 16,
          child: isCalling ? CallingTime(startTime: answerTime) : Container(),
        ),
        const SizedBox(
          height: 16,
        ),
        _buildCallingBtnList(),
        if (isVideoCall)
          const SizedBox(
            height: 44,
          ),
        _buildSignlingBtn(),
        const SizedBox(
          height: 10,
        ),
      ],
    );
  }

  Widget _buildInviterInfo() {
    final faceUrl = widget.inviterUserInfo.faceUrl;
    final inviterName = widget.inviterUserInfo.nickName != ""
        ? widget.inviterUserInfo.nickName
        : widget.inviterUserInfo.userID;
    BoxDecoration selfPortraitDecorat = BoxDecoration(
      borderRadius: BorderRadius.circular(8),
      image: DecorationImage(
        image: NetworkImage(faceUrl ?? ""),
        fit: BoxFit.cover,
      ),
    );

    return Column(
      // mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Padding(
          padding: const EdgeInsets.symmetric(vertical: 16),
          child: Text(
            TIM_t("等待接听"),
            style: const TextStyle(fontSize: 16, color: Colors.white),
          ),
        ),
        Container(
            height: 110,
            width: 110,
            child: Container(),
            decoration: selfPortraitDecorat),
        Padding(
          padding: const EdgeInsets.symmetric(vertical: 16),
          child: Text(
            inviterName ?? "",
            style: const TextStyle(fontSize: 24, color: Colors.white),
          ),
        ),
        Text(
          TIM_t("邀请你加入多人通话"),
          style: const TextStyle(color: Color(0xFF999999)),
        ),
        const SizedBox(
          height: 64,
        ),
        Text(
          TIM_t("参与通话的还有:"),
          style: const TextStyle(color: Color(0xFF999999)),
        ),
        Container(
          margin: const EdgeInsets.only(top: 16),
          child: Wrap(
            spacing: 5,
            runSpacing: 5,
            children:
                List.generate(widget.inviteUsersInfoList.length, ((index) {
              final inviteItem = widget.inviteUsersInfoList[index];
              final faceUrl = inviteItem.faceUrl ?? "";
              return Container(
                height: 40,
                width: 40,
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(8),
                  image: DecorationImage(
                    image: NetworkImage(faceUrl == ""
                        ? "https://imgcache.qq.com/qcloud/public/static//avatar3_100.20191230.png"
                        : faceUrl),
                    fit: BoxFit.cover,
                  ),
                ),
              );
            })),
          ),
        )
      ],
    );
  }

  _handleOnUserVideoAvailable(OnUserVideoAvailableParams params) async {
    final userID = params.userID;
    final isAvailable = params.available;
    userInfoList = userInfoList.map((e) {
      if (e.userInfo.userID == userID) {
        e.isMuteVideo = !isAvailable;
      }
      return e;
    }).toList();
    setState(() {});
  }

  _handleUserLeaveAndJoin(params, bool isJoin) {
    if (!isCalling && isJoin) {
      isCalling = true;
      answerTime = DateTime.now();
    }
    final userID = params;
    if (isJoin) {
      userInfoList = userInfoList.map((e) {
        if (e.userInfo.userID == userID) {
          e.isJoined = isJoin;
        }
        return e;
      }).toList();
    } else {
      userInfoList.removeWhere((element) => element.userInfo.userID == userID);
    }

    setState(() {});
  }

  _handleCallingCance() {
    stopCameraAndFinish();
  }

  _registerListener() {
    _onRtcListener = TUICallingListener(
        onCallEnd: () {
          _handleCallingCance();
        },
        onCallingTimeout: (params) {
          final timeoutUserList = params.userIDList;
          for (var user in timeoutUserList) {
            userInfoList
                .removeWhere((element) => element.userInfo.userID == user);
          }
          setState(() {});
        },
        onReject: (params) {
          final rejectUser = params.userID;
          userInfoList
              .removeWhere((element) => element.userInfo.userID == rejectUser);
          setState(() {});
        },
        onUserLeave: (params) {
          final userID = params.userID;
          _handleUserLeaveAndJoin(userID, false);
        },
        onUserEnter: (params) {
          final userID = params.userID;
          _handleUserLeaveAndJoin(userID, true);
        },
        onNoResp: (params) {
          final noResponseUser = params.userID;
          userInfoList.removeWhere(
              (element) => element.userInfo.userID == noResponseUser);
          setState(() {});
        },
        onLineBusy: (params) {
          final onLineBusyUser = params.userID;
          userInfoList.removeWhere(
              (element) => element.userInfo.userID == onLineBusyUser);
          setState(() {});
        },
        onCallingCancel: () {
          _handleCallingCance();
        },
        onEnterRoom: (time) {},
        onUserVideoAvailable: (params) {
          _handleOnUserVideoAvailable(params);
        });
    _tRTCCallingService.registerListener(_onRtcListener);
  }

  _initTRTC() async {
    _tRTCCallingService = await TRTCCalling.sharedInstance();
    _registerListener();
    _buildCallingMemberInfo();

    /// 如果是自己发出的邀请
    if (widget.isInitiationCalling) {
      Future.delayed(Duration.zero, () {
        _groupCall();
      });
    }
  }

  stopCameraAndFinish() {
    _tRTCCallingService.setMicMute(true);
    _tRTCCallingService.closeCamera();
    Future.delayed(const Duration(seconds: 1), () {
      widget.close();
    });
  }

  _groupCall() async {
    final userIDList =
        widget.inviteUsersInfoList.map((e) => e.userID ?? "").toList();
    final callType = widget.callScenes == CallingScenes.Video
        ? TRTCCalling.typeVideoCall
        : TRTCCalling.typeAudioCall;
    ActionCallback res = await _tRTCCallingService.groupCall(userIDList, callType, widget.groupID);
    if(res.code!=0){
                Fluttertoast.showToast(
                  msg: res.desc,
                  gravity: ToastGravity.BOTTOM,
                  timeInSecForIosWeb: 3,
                  textColor: Colors.white,
                  backgroundColor: Colors.black,
      );
      await Future.delayed(const Duration(seconds: 3));
      stopCameraAndFinish();
                //挂断
    }
  }

  @override
  void initState() {
    _initTRTC();
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
    _tRTCCallingService.unRegisterListener(_onRtcListener);
    // _tRTCCallingService.hangup();
  }

  @override
  Widget build(BuildContext context) {
    final isRing = !(widget.isInitiationCalling || isCalling);
    return Scaffold(
      body: WillPopScope(
          child: Container(
            color: const Color(0xFF444444),
            child: SafeArea(
              child: Column(
                children: [
                  SizedBox(
                      height: isRing ? 300 : 100 * 3,
                      width: MediaQuery.of(context).size.width,
                      child:
                          !isRing ? _buildScreenList() : _buildInviterInfo()),
                  Expanded(
                    child: Padding(
                      padding: const EdgeInsets.symmetric(horizontal: 0),
                      child: _buildBottomBtn(),
                    ),
                  )
                ],
              ),
            ),
          ),
          onWillPop: () async {
            _exitRoom();
            return true;
          }),
    );
  }
}
