import 'package:flutter/material.dart';
import 'package:tencent_im_base/tencent_im_base.dart';

import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_call_conv_type.dart';
import 'package:tim_ui_kit_calling_plugin/enum/tim_uikit_trtc_calling_scence.dart';
import 'package:tim_ui_kit_calling_plugin/view/c2c/tim_uikit_trtc_c2c_calling.dart';
import 'package:tim_ui_kit_calling_plugin/view/dragable.dart';

import 'package:tim_ui_kit_calling_plugin/view/group/tim_uikit_group_call.dart';
import 'package:wakelock/wakelock.dart';

class TIMUIKitTRTCContainer extends StatefulWidget {
  final bool isInitiationCalling;
  final bool enableFloatingWindow;
  final ConvType convType;
  final V2TimUserFullInfo inviterUserInfo;
  final List<V2TimUserFullInfo> inviteUsersInfoList;
  final CallingScenes callScenes;
  final String? groupID;
  final OfflinePushInfo? offlinePushInfo;
  final Function close;
  const TIMUIKitTRTCContainer({
    // 判断群组通话还是c2c通话
    required this.convType,
    // 邀请人信息
    required this.inviterUserInfo,
    // 被邀请人信息,c2c情况下 只取第一个
    required this.inviteUsersInfoList,

    /// 语音通话 :CallingScenes.Audio
    /// 视频通话:CallingScenes.Video
    required this.callScenes,
    // 是否主动发起通话
    required this.isInitiationCalling,
    this.groupID,
    Key? key,
    required this.close,
    required this.enableFloatingWindow,
    this.offlinePushInfo,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _TIMUIKitTRTCContainerState();
}

class _TIMUIKitTRTCContainerState extends State<TIMUIKitTRTCContainer> {
  bool isMinized = false;

  @override
  void initState() {
    final isVideoCall = widget.callScenes == CallingScenes.Video;
    // 视频通话保持屏幕常亮
    Wakelock.toggle(enable: isVideoCall);
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
    Wakelock.toggle(enable: false);
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      color: Colors.transparent,
      child: IndexedStack(
        index: isMinized ? 1 : 0,
        children: [
          Stack(
            children: [
              Positioned(
                top: 0,
                right: 0,
                left: 0,
                bottom: 0,
                child: TIMUIKitTRTC(
                  callScenes: widget.callScenes,
                  close: widget.close,
                  inviterUserInfo: widget.inviterUserInfo,
                  inviteUsersInfoList: widget.inviteUsersInfoList,
                  isInitiationCalling: widget.isInitiationCalling,
                  convType: widget.convType,
                  groupID: widget.groupID,
                  offlinePushInfo: widget.offlinePushInfo,
                ),
              ),
              if (widget.enableFloatingWindow)
                Positioned(
                    top: 50,
                    left: 10,
                    child: InkWell(
                      onTap: (() {
                        isMinized = true;
                        setState(() {});
                      }),
                      child: Image.asset(
                        'images/minization-window.png',
                        package: 'tim_ui_kit_calling_plugin',
                        height: 32,
                      ),
                    ))
            ],
          ),
          DragArea(
              child: InkWell(
                onTap: () {
                  isMinized = false;
                  setState(() {});
                },
                child: Container(
                  decoration: BoxDecoration(
                      color: Colors.white,
                      borderRadius: BorderRadius.circular(15),
                      border: Border.all(color: const Color(0xFFE5E6E9), width: 7)),
                  padding: const EdgeInsets.all(12),
                  child: Column(
                    children: [
                      Image.asset(
                        'images/calling.png',
                        package: 'tim_ui_kit_calling_plugin',
                        height: 32,
                      ),
                      Text(
                        TIM_t("正在通话"),
                        style: const TextStyle(color: Color(0xFF147AFF)),
                      )
                    ],
                  ),
                ),
              ))
        ],
      ),
    );
  }
}

class TIMUIKitTRTC extends StatelessWidget {
  final bool isInitiationCalling;
  final ConvType convType;
  final V2TimUserFullInfo inviterUserInfo;
  final List<V2TimUserFullInfo> inviteUsersInfoList;
  final CallingScenes callScenes;
  final String? groupID;
  final OfflinePushInfo? offlinePushInfo;
  final Function close;

  const TIMUIKitTRTC({
    // 判断群组通话还是c2c通话
    required this.convType,
    // 邀请人信息
    required this.inviterUserInfo,
    // 被邀请人信息,c2c情况下 只取第一个
    required this.inviteUsersInfoList,

    /// 语音通话 :CallingScenes.Audio
    /// 视频通话:CallingScenes.Video
    required this.callScenes,
    // 是否主动发起通话
    required this.isInitiationCalling,
    this.offlinePushInfo,
    this.groupID,
    Key? key,
    required this.close,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return convType == ConvType.c2c
        ? TIMUIKitTRTCc2cCalling(
            convType: convType,
            inviteUsersInfoList: inviteUsersInfoList,
            inviterUserInfo: inviterUserInfo,
            callScenes: callScenes,
            isInitiationCalling: isInitiationCalling,
            offlinePushInfo: offlinePushInfo,
            close: close)
        : TIMUIKitGroupCalling(
            convType: convType,
            inviteUsersInfoList: inviteUsersInfoList,
            inviterUserInfo: inviterUserInfo,
            callScenes: callScenes,
            isInitiationCalling: isInitiationCalling,
            groupID: groupID,
            close: close,
          );
  }
}
